package com.link.core.util;

import com.link.core.basic.model.BasicModel;
import com.link.core.basic.service.BasicService;
import com.link.core.cllog.LCLogger;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

public class ReflectUtil {
   private static final Logger logger = LogManager.getLogger(ReflectUtil.class);

   public static <T> Object invokeGetter(T target, String fieldName) throws NoSuchMethodException, SecurityException, IllegalAccessException, IllegalArgumentException, InvocationTargetException {
      String methodName = "get" + StringUtils.firstCharUpperCase(fieldName);
      Method method = target.getClass().getMethod(methodName);
      return method.invoke(target);
   }

   public static <T> Object invokeGetter(T target, String fieldName, Class<?> clazz) throws SecurityException, IllegalAccessException, IllegalArgumentException, InvocationTargetException, NoSuchMethodException {
      try {
         String methodName = "get" + StringUtils.firstCharUpperCase(fieldName);
         Method method = target.getClass().getMethod(methodName);
         return method.invoke(target);
      } catch (NoSuchMethodException var6) {
         NoSuchMethodException e = var6;
         if (BasicModel.class.equals(clazz)) {
            throw e;
         } else {
            Class<?> superClass = null;
            if (clazz.isInterface()) {
               Class<?>[] interfaces = clazz.getInterfaces();
               if (interfaces != null && interfaces.length == 1) {
                  superClass = interfaces[0];
               }
            } else {
               superClass = clazz.getSuperclass();
            }

            if (superClass != null && superClass != Object.class) {
               return invokeGetter(target, fieldName, superClass);
            } else {
               throw e;
            }
         }
      }
   }

   public static <T> void invokeSetter(T target, String fieldName, Object args) throws NoSuchFieldException, SecurityException, NoSuchMethodException, IllegalAccessException, IllegalArgumentException, InvocationTargetException {
      String methodName = "set" + StringUtils.firstCharUpperCase(fieldName);
      Class<?> clazz = target.getClass();
      Field field = clazz.getDeclaredField(fieldName);
      Method method = clazz.getMethod(methodName, field.getType());
      method.invoke(target, args);
   }

   public static <T> void invokeSetter(T target, String fieldName, Object args, Class<?> clazz) throws NoSuchFieldException, SecurityException, IllegalAccessException, IllegalArgumentException, InvocationTargetException, InstantiationException {
      try {
         String methodName = "set" + StringUtils.firstCharUpperCase(fieldName);
         Field field = clazz.getDeclaredField(fieldName);
         Method method = clazz.getMethod(methodName, field.getType());
         method.invoke(target, args);
      } catch (NoSuchFieldException var7) {
         NoSuchFieldException e = var7;
         if (BasicModel.class.equals(clazz)) {
            throw e;
         }

         Class<?> superClass = null;
         if (clazz.isInterface()) {
            Class<?>[] interfaces = clazz.getInterfaces();
            if (interfaces != null && interfaces.length == 1) {
               superClass = interfaces[0];
            }
         } else {
            superClass = clazz.getSuperclass();
         }

         if (superClass == null || superClass == Object.class) {
            throw e;
         }

         invokeSetter(target, fieldName, args, superClass);
      } catch (NoSuchMethodException var8) {
      }

   }

   public static List<Field> getAllFields(Class<?> clazz) throws Exception {
      List<Field> fieldList = new ArrayList();

      for(Class<?> tempClass = clazz; tempClass != null; tempClass = tempClass.getSuperclass()) {
         fieldList.addAll(Arrays.asList(tempClass.getDeclaredFields()));
      }

      return fieldList;
   }

   public static Field getFieldByName(Class<?> clazz, String fieldName) throws NoSuchFieldException {
      Field declaredField = null;
      Class<?> tempClass = clazz;

      while(tempClass != null) {
         try {
            declaredField = tempClass.getDeclaredField(fieldName);
            return declaredField;
         } catch (NoSuchFieldException var5) {
            tempClass = tempClass.getSuperclass();
         }
      }

      throw new NoSuchFieldException(fieldName);
   }

   public static Field getField(Class<?> parameterObjectClass, String propertyName) throws Exception {
      Field field = null;
      List<Field> fieldList = getAllFields(parameterObjectClass);
      Iterator var4 = fieldList.iterator();

      while(var4.hasNext()) {
         Field f = (Field)var4.next();
         if (propertyName.equals(f.getName())) {
            field = f;
         }
      }

      return field;
   }

   public static Method getMethod(Class<?> clz, String methodName, Class<?>... parameterTypes) {
      Method method = null;

      try {
         if (clz == BasicService.class) {
            Method[] var4 = clz.getMethods();
            int var9 = var4.length;

            for(int var10 = 0; var10 < var9; ++var10) {
               Method item = var4[var10];
               if (methodName.equals(item.getName())) {
                  return item;
               }
            }

            return null;
         }

         method = clz.getMethod(methodName, parameterTypes);
      } catch (NoSuchMethodException var8) {
         Class<?> superClass = null;
         if (clz.isInterface()) {
            Class<?>[] interfaces = clz.getInterfaces();
            if (interfaces != null && interfaces.length == 1) {
               superClass = interfaces[0];
            }
         } else {
            superClass = clz.getSuperclass();
         }

         if (superClass == null || superClass == Object.class) {
            return null;
         }

         method = getMethod(superClass, methodName, parameterTypes);
      }

      return method;
   }

   public static Integer getNumByFiledName(Object obj, String filedName) throws Exception {
      Field[] fields = obj.getClass().getDeclaredFields();
      Integer num = null;
      Field[] var4 = fields;
      int var5 = fields.length;

      for(int var6 = 0; var6 < var5; ++var6) {
         Field field = var4[var6];
         if (field.getName().equals(filedName)) {
            num = field.getInt(obj);
         }
      }

      return num;
   }

   public static <T> void invokeSetterNew(T target, String fieldName, Object arg) {
      PropertyDescriptor descriptor = null;
      Object obj = null;

      try {
         descriptor = new PropertyDescriptor(fieldName, target.getClass());
         Method writeMethod = descriptor.getWriteMethod();
         Class<?> paramClazz = writeMethod.getParameterTypes()[0];
         writeMethod.invoke(target, arg);
      } catch (Exception var7) {
         logger.error("调用setter方法出现异常，属性名为：" + fieldName);
         LCLogger.error().withMessageKey("ReflectUtil.invokeSetter").withMessage("调用setter方法出现异常，属性名为：" + fieldName).flush();
      }

   }

   public static <T> Object invokeGetterNew(T target, String fieldName) {
      PropertyDescriptor descriptor = null;
      Object obj = null;

      try {
         descriptor = new PropertyDescriptor(fieldName, target.getClass());
         Method readMethod = descriptor.getReadMethod();
         obj = readMethod.invoke(target);
      } catch (Exception var5) {
         Exception e = var5;
         logger.error("调用getter方法出现异常，属性名为：" + fieldName + e.getMessage());
         LCLogger.error().withMessageKey("ReflectUtil.invokeGetter").withMessage("调用getter方法出现异常，属性名为：" + fieldName).flush();
      }

      return obj;
   }

   public static void assignValuesAll(Object source, Object... targets) throws Exception {
      Object[] var2 = targets;
      int var3 = targets.length;

      for(int var4 = 0; var4 < var3; ++var4) {
         Object item = var2[var4];
         List<Field> sourceFields = getAllFields(source.getClass());
         Iterator var7 = sourceFields.iterator();

         while(var7.hasNext()) {
            Field field = (Field)var7.next();
            if (!Modifier.isFinal(field.getModifiers())) {
               Object value = invokeGetterNew(source, field.getName());
               invokeSetterNew(item, field.getName(), value);
            }
         }
      }

   }
}
